@@ -57,8 +57,9 @@ class AFPEntitiesProcessor(ProcessorBase):
         for document in documents:
             with add_logging_context(docid=document.identifier):
                 if document.annotations:
-                    mark_whitelisted(document, params.white_label)
-                    ann_groups = group_annotations(document, by_lexicon)
+                    annotations = [a for a in document.annotations if a.labelName not in ["witness", 'loc_org', 'signature']]
+                    mark_whitelisted(annotations, params.white_label)
+                    ann_groups = group_annotations(annotations, by_lexicon)
                     # 1. Compute document fingerprint
                     fingerprints = compute_fingerprint(ann_groups)
                     if params.as_altText is not None and len(params.as_altText):
@@ -129,8 +130,8 @@ def compute_fingerprint(ann_groups):
     return fingerprints
 
 
-def mark_whitelisted(document, white_label):
-    for a in document.annotations:
+def mark_whitelisted(annotations, white_label):
+    for a in annotations:
         if (
             a.labelName == white_label
         ):  # Consider whitelisted terms as entities coming from the model
@@ -209,12 +210,12 @@ def consolidate_and_link(
     return conso_anns
 
 
-def group_annotations(doc: Document, keyfunc):
+def group_annotations(annotations, keyfunc):
     def get_sort_key(a: Annotation):
         return a.end - a.start, -a.start
 
     groups = defaultdict(RangeMap)
-    for k, g in groupby(sorted(doc.annotations, key=keyfunc), keyfunc):
+    for k, g in groupby(sorted(annotations, key=keyfunc), keyfunc):
         sorted_group = sorted(g, key=get_sort_key, reverse=True)
         groups[k] = RangeMap((a.start, a.end, a) for a in sorted_group)
     return groups
@@ -249,10 +250,11 @@ def by_label(a: Annotation):
 def one_match(a: Annotation, matches: RangeMap):
     match = None
     perfect = False
-    if matches and len(matches) == 1:
-        match = matches.get(a.start) or matches.get(a.end)
-        if match:
+    if matches and len(matches) >= 1:
+        for match in matches.values():
             perfect = a.start == match.start and a.end == match.end
+            if perfect:
+                break
     return perfect, match
 
 
@@ -328,7 +330,12 @@ def validate_wiki_type(w: Annotation, gname: str):
                                 "Q20746389",
                                 "Q48204",
                                 "Q207320",
-                                "Q7278"
+                                "Q7278",
+                                "Q875538",
+                                "Q3918",
+                                "Q14350",
+                                "Q15265344",
+                                "Q11033"
                             ]
                         ),
                         fingerprint,
